// vid_vgl.c support for FreeBSD's libvgl native VGA graphics.

#include <vgl.h>
#include <machine/console.h>

#include "quakedef.h"
#include "d_local.h"
viddef_t vid; 		// global video state
unsigned short d_8to16table[256];

int	verbose;
void    *vid_surfcache;
int     vid_surfcachesize;
int     vid_buffersize;

int             VGA_width, VGA_height, VGA_rowbytes, VGA_bufferrowbytes, VGA_planar;
byte    *VGA_pagebase;
static  byte current_palette[768];

cvar_t _windowed_mouse = {"_windowed_mouse", "1", true};
cvar_t  m_filter = {"m_filter","0"};

void	(*vid_menudrawfn)(void);
void	(*vid_menukeyfn)(int key);
//void	VID_MenuKey (int key);

// ========================================================================
// Tragic death handler
// ========================================================================

void TragicDeath(int signal_num)
{
	VGLEnd();
	Sys_Error("This death brought to you by the number %d\n", signal_num);
}

// ========================================================================
// VID_Gamma_f Keybinding command
// ========================================================================

void VID_Gamma_f (void)
{
    float gamma, f, inf;
    unsigned char palette[768];
    int i;

    if (Cmd_Argc () == 2) {
        gamma = Q_atof (Cmd_Argv(1));
        for (i=0 ; i<768 ; i++) {
            f = pow ( (host_basepal[i]+1)/256.0 , gamma );
            inf = f*255 + 0.5;
            if (inf < 0)
                inf = 0;
            if (inf > 255)
                inf = 255;
            palette[i] = inf;
        }
        VID_SetPalette (palette);
        vid.recalc_refdef = 1;	// force a surface cache flush
    }
}

// ========================================================================
// Called at startup to set up translation tables, takes 256 8 bit RGB values
// the palette data will go away after the call, so it must be copied off if
// the video driver will need it again
// ========================================================================

void VID_Init(unsigned char *palette)
{
	verbose=COM_CheckParm("-verbose");
	srandom(getpid());

// open the display
	VGLInit(SW_VGA_CG320);
	VGLKeyboardInit(VGL_CODEKEYS);
	vid.width = VGLDisplay->Xsize;
	vid.rowbytes = VGLDisplay->Xsize;
	vid.height = VGLDisplay->Ysize;
	vid.aspect = ((float)vid.height / (float)vid.width) * (320.0 / 240.0);
	vid.colormap = host_colormap;
	vid.fullbright = 256 - LittleLong (*((int *)vid.colormap + 2048));
	vid.maxwarpwidth = WARP_WIDTH;
	vid.maxwarpheight = WARP_HEIGHT;
	vid.numpages = 1;
//	vid.cbits = VID_CBITS;
//	vid.grades = VID_GRADES;
	vid.direct = VGLDisplay->Bitmap;;

	vid.conwidth = vid.width;
	vid.conrowbytes = vid.rowbytes;
	vid.conheight = vid.height;
#if 0
	vid_buffersize = vid.rowbytes*(vid.height+1)*sizeof(*d_pzbuffer) +
			 vid.rowbytes*vid.height;
	vid_surfcachesize = D_SurfaceCacheForRes(vid.width, vid.height);
	vid_buffersize += vid_surfcachesize;

        d_pzbuffer = Hunk_HighAllocName(vid_buffersize, "video");
        if (d_pzbuffer == NULL)       
            Sys_Error("Not enough memory for video mode\n");

        vid_surfcache = (byte *)d_pzbuffer + vid.width*vid.height;
        D_InitCaches(vid_surfcache, vid_surfcachesize);
	vid.buffer = (void *)( (byte *)vid_surfcache + vid_surfcachesize);    
	vid.conbuffer = vid.buffer;
#else

	vid.buffer = malloc(vid.rowbytes*vid.height);
	vid.conbuffer = vid.buffer;
	VGA_pagebase = vid.buffer;
        d_pzbuffer = malloc(vid.rowbytes*vid.height*sizeof(*d_pzbuffer));
	vid_surfcache = malloc(600*1024);
        D_InitCaches(vid_surfcache, 600*1024);
#endif
	VID_SetPalette(palette);
}

void VID_ShiftPalette(unsigned char *p)
{
	VID_SetPalette(p);
}

void VID_SetPalette(unsigned char *palette)
{

	char red[256], green[256], blue[256];
	int i;

	if (palette != current_palette)
	memcpy(current_palette, palette, 768);
	for (i=0 ; i<256 ; i++) {
		red[i] = palette[i*3]>>2;
		green[i] = palette[i*3+1]>>2;
		blue[i] = palette[i*3+2]>>2;
	}
	VGLSetPalette(red, green, blue);
}

// Called at shutdown

void	VID_Shutdown (void)
{
	Con_Printf("VID_Shutdown\n");
	VGLEnd();
}

// flushes the given rectangles from the view buffer to the screen
void VID_Update (vrect_t *rects)
{
    int ycount;
    int offset;
 
    while (rects) {
        ycount = rects->height;
        offset = rects->y * vid.rowbytes + rects->x;
        while (ycount--) {
            memcpy(VGLDisplay->Bitmap+offset, vid.buffer+offset, rects->width);
            offset += vid.rowbytes;
        }
        rects = rects->pnext;
    }       
}

static int dither;

void VID_DitherOn(void)
{
    if (!dither)
    {
	vid.recalc_refdef = 1;
        dither = 1;
    }
}

void VID_DitherOff(void)
{
    if (dither)
    {
	vid.recalc_refdef = 1;
        dither = 0;
    }
}

int Sys_OpenWindow(void)
{
	return 0;
}

void Sys_EraseWindow(int window)
{
}

void Sys_DrawCircle(int window, int x, int y, int r)
{
}

void Sys_DisplayWindow(int window)
{
}

//=============================================================================

byte        scantokey[128] = 
					{ 
//  0           1       2       3       4       5       6       7 
//  8           9       A       B       C       D       E       F 
	0  ,    27,     '1',    '2',    '3',    '4',    '5',    '6', 
	'7',    '8',    '9',    '0',    '-',    '=',    K_BACKSPACE, 9, // 0 
	'q',    'w',    'e',    'r',    't',    'y',    'u',    'i', 
	'o',    'p',    '[',    ']',    13 ,    K_CTRL,'a',  's',      // 1 
	'd',    'f',    'g',    'h',    'j',    'k',    'l',    ';', 
	'\'' ,    '`',    K_SHIFT,'\\',  'z',    'x',    'c',    'v',      // 2 
	'b',    'n',    'm',    ',',    '.',    '/',    K_SHIFT,'*', 
	K_ALT,' ',   0  ,    K_F1, K_F2, K_F3, K_F4, K_F5,   // 3 
	K_F6, K_F7, K_F8, K_F9, K_F10,0  ,    0  , K_HOME, 
	K_UPARROW,K_PGUP,'-',K_LEFTARROW,'5',K_RIGHTARROW,'+',K_END, //4 
	K_DOWNARROW,K_PGDN,K_INS,K_DEL,0,0,             0,              K_F11, 
	K_F12,0  ,    0  ,    0  ,    0  ,    0  ,    0  ,    K_UPARROW,  // 5 
	0  ,K_LEFTARROW,K_RIGHTARROW,    0  ,K_DOWNARROW,    0  ,    0  ,    0,
	0  ,    0  ,    0  ,    0  ,    0  ,    0  ,    0  ,    0  ,      // 6 

	0  ,    0  ,    0  ,    0  ,    0  ,    0  ,    0  ,    0, 
	0  ,    0  ,    0  ,    0  ,    0  ,    0  ,    0  ,    0         // 7 
					}; 

// get events from the keyboard
void Sys_SendKeyEvents(void)
{
  int key = 0;

  while ((key=VGLKeyboardGetCh()) != 0) {
    Key_Event(scantokey[key&0x7f], (key&0x80 ? 0 : 1));
  }
}

void D_BeginDirectRect (int x, int y, byte *pbitmap, int width, int height)
{
// direct drawing of the "accessing disk" icon isn't supported
}

void D_EndDirectRect (int x, int y, int width, int height)
{
// direct drawing of the "accessing disk" icon isn't supported
}

void IN_Move(usercmd_t *cmd) {

	return;
}

void IN_Commands (void) {
	return;
}

void VID_LockBuffer (void)
{
}

void VID_UnlockBuffer (void)
{
}

void IN_Init(void)
{
}

void IN_Shutdown(void) {

}
